import pkg_resources

from mako.template import Template

import p2.agent
from p2.store.client import Client
from p2.logging import HttpLogger
from p2.tools.dictionary import get_recursively
from p2.slack import Webhook

class BaseRunner:
    """A common base class to run experiments/templates."""

    def __init__(self, p2_store_url, filename, exp_id, campaign_id:dict=None):

        if not (p2_store_url and filename and exp_id):
            raise Exception("Not all parameters passed")

        self.filename = filename
        self.exp_id = exp_id
        self.campaign_id = campaign_id
        self.p2_store_url = p2_store_url
        self.client = Client(p2_store_url)
        log_obj = HttpLogger(
            silent=False,
            p2_store_url=p2_store_url,
            name=exp_id
        )
        self.log = log_obj.getLogger()

        self.log.info(f"Session: {exp_id}, Filename: {filename}")

        sessions = self.client.get_sessions(exp_id=self.exp_id)
        if ((sessions is None) or (len(sessions) == 0)):
            self.log.error("No session found")
            raise Exception("No session found")
        self.session = sessions[0]
        if self.session['status'] != 'scheduled':
            self.log.error("Session not in scheduled status, aborting...")
            raise Exception("Session not in scheduled status, aborting...")


    def run(self):
        """ Implemented by single driver """
        pass


    def notify_slack(self,
                     resource: dict,
                     journal: dict
                     ):
        """
        Implements the logic to notify the team via slack in case of errors.
        Only meaningful in application (Kubernetes) level tests.
        """
        try:
            all_exceptions = get_recursively(journal, 'exception')
            ns = get_recursively(journal, 'ns') # this should be the namespace...
            if len(ns) > 0:
                # okay, ideally we should retrieve the namespace for the right 
                # context, which should come from the campaign targets
                # but we trust the same team is behind a given namespace
                # no matter on which context 
                annotations = self.client.get_namespace_annotations(
                    ns=ns[0],
                    context=resource['tag'])
                slack_text = annotations.get('slack')
                slack_channel = slack_text.split(' ')[0]
                # TODO
                # this is not the best way to retrieve a package data file,
                # but "technically" p2.agent is a namespace package and
                # hence apparently the usual pkg_resources.resource_filename
                # func doesn't work...
                # template_file = pkg_resources.resource_filename('p2.agent', 'data/slack_message.mako')
                template_file = p2.agent.__path__[-1]+'/data/slack_message.mako'
                if template_file is None:
                    raise Exception("Error, template not found in package")
                template = Template(filename=template_file)
                params = {
                    'channel': slack_channel,
                    'app': 'the persistence-slack-app App',
                    'exp_id': self.exp_id,
                    'start_time': journal['start'],
                    'end_time': journal['end'],
                    'k8s_context': resource['tag'],
                    'resource': resource['name'],
                    'exceptions': all_exceptions[0],
                    'me': '@marco.masetti'
                }
                message = template.render(**params)
                hook = Webhook()
                self.log.debug(f"Slack channel: {slack_channel}")
                self.log.debug(f"Message: {message}")
                hook.send_message_to_channel(slack_channel, message)
            else:
                self.log.warn("""Can't find a namespace definition in the test
                            journal, maybe the target is not a Kubernetes resource ?""")
        except Exception as e:
            self.log.error(f"Error trying to notify team via slack: {e}")
