import json
import subprocess
import requests
import logging
import regex as rx
from datetime import datetime
from croniter import croniter

import http.client as http_client
http_client.HTTPConnection.debuglevel = 1

from p2.store.client import Client
from p2.agent.baseRunner import BaseRunner
from p2.tools.dictionary import get_recursively
from p2.slack import Webhook

class Runner(BaseRunner):
    """A class to run an experiment

    Usage:
        from p2.tools.driver import get_driver

        runner = get_driver(
            module='p2.agent',
            driver='experiment',
            class_name='Runner',
            p2_store_url : p2.store.server url
            filename     : experiment filename to run
            exp_id       : experiment session
            )

        runner.run()
    """

    def run(self):
        experiment_url = self.p2_store_url+'/experiment/'+self.filename
        process = subprocess.run(["chaos", "run", experiment_url], capture_output=True)

        # get process output
        output = process.stderr.decode()

        self.log.debug("---------- Process stderr --------------")
        self.log.debug(output)

        journal = dict()
        # get start_time, end_time from journal.log
        with open('./journal.json') as f:
            journal = json.load(f)

        if 'status' not in journal:
            self.log.error("Something went wrong here, no experiment journal")
            raise Exception("Something went wrong here, no experiment journal")

        self.log.debug(f"Updating session {self.session['exp_id']}")
        self.log.debug(f"Status: {journal['status']}")
        self.log.debug(f"Start time: {journal['start']}")
        self.log.debug(f"End time: {journal['end']}")

        cron = croniter(self.session['schedule'], datetime.now())
        #
        # status is set to completed only if no action ended in failed state...

        status = journal["status"]
        for activity in journal['run']:
            if activity['status'] == 'failed':
                status = 'failed'
                break

        id = self.client.update_session(self.exp_id, {
            'status'    : status,
            'start_time': journal["start"],
            'end_time'  : journal["end"],
            'next_run_time' : str(cron.get_next(datetime))
            })
        if id != self.exp_id:
            self.log.error("Error updating session")

        # store experiment output and journal as logs
        with open('./chaostoolkit.log') as f:
            log_text = f.read()

            self.client.update_log(self.exp_id, {
                "log_type" : 2,
                "log_text" : log_text,
                })

            self.client.update_log(self.exp_id, {
                "log_type" : 3,
                "log_text" : json.dumps(journal, sort_keys=True)
                })

            self.client.update_log(self.exp_id, {
                "log_type" : 1,
                "log_text" : output
                })

        try:
            resource = self.client.select(
                table='resources',
                filter={'id': self.session['resource_id']}
            )[0]

            if resource['layer'] == 'k8' and status == 'failed':
                self.notify_slack(resource=resource, journal=journal)
        except Exception as e:
            self.log.error(f"Error trying to notify team via slack: {e}")
