import os
import logging

from fastapi import FastAPI, Request
from fastapi.responses import JSONResponse
from logging.handlers import RotatingFileHandler
from pprint import pformat

import p2.driver.k8s
from p2.driver.k8s import k8s, jobs
from p2.agent.cronjobs import create_cronjob
from p2.agent.config import Config

__version__ = '0.5.13'

context = os.getenv('KUBERNETES_CONTEXT') or k8s.get_current_context()['context']['cluster']
config = Config(context)

log = logging.getLogger(__name__)
log.setLevel(config.logger.get('log_level', 'INFO'))

handler = RotatingFileHandler(
    config.logger.get('filename', 'server.log'),
    int(config.logger.get('max_bytes', 1000000)),
    int(config.logger.get('backups', 5))
    )
# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# add formatter to ch
handler.setFormatter(formatter)
log.addHandler(handler)
log.info("Using p2.driver.k8s version: {}".format(p2.driver.k8s.__version__))
log.info(f"Using p2.agent version: {__version__}")
log.info(f"Working on context {context} ...")

settings = {}

app = FastAPI()

@app.on_event("startup")
async def config_settings():
    image = config.cronjobs.get('image')
    if image is None:
        log.error("ERROR: No cronjob image  defined in config file")
        exit()

    settings['image']             = image
    settings['command']           = config.cronjobs.get('command')
    settings['namespace']         = config.k8s.get('namespace', 'default')
    settings['service_account']   = config.cronjobs.get('service_account')
    settings['p2_store_endpoint'] = config.cronjobs.get('p2_store_endpoint',
            'http://localhost:6000')

    log.info("Using settings: {}".format(pformat(settings)))

@app.get('/agent/version')
def getVersion():
    return __version__

#-------------------------------------------------------------------------
# Cronjobs management
#-------------------------------------------------------------------------
@app.get('/agent/k8s/cronjobs/{context}')
def getCronjobs(context: str):
    log.info(f"Getting cronjobs for context: {context}")
    k8s_obj = k8s.Cron(context=context, namespace=settings['namespace'])
    cronjobs = k8s_obj.get_cronjobs()
    return cronjobs


@app.post('/agent/k8s/cronjob/{context}/{exp_id}', status_code=201)
async def createCronjob(context:str, exp_id:str, request: Request):
    """Creates a new cronjob with id as passed

    Payload: {
        filename    : the experiment filename
        driver      : experiment/template
        crontab     : the cronjob crontab
        campaign_id : the campaign id if any
    }

    """

    data = await request.json()
    if 'crontab' not in data:
        log.error("Error: no crontab key in payload")
        return JSONResponse(status_code=500)

    if 'driver' not in data:
        data['driver'] = 'experiment'

    command = settings['command']
    log.info(f"Creating cronjob {exp_id}, command {command}")
    log.debug("""Request body: {}""".format(data))

    cmd_array=list()
    if data.get('campaign_id'):
        cmd_array = [
            command,
            '--p2_store_url',
            settings['p2_store_endpoint'],
            '--filename',
            data['filename'],
            '--driver',
            data['driver'],
            '--session_id',
            exp_id,
            '--campaign_id',
            str(data['campaign_id'])
            ]
    else:
        cmd_array = [
            command,
            '--p2_store_url',
            settings['p2_store_endpoint'],
            '--filename',
            data['filename'],
            '--driver',
            data['driver'],
            '--session_id',
            exp_id
        ]
    try:
        cronjob_name = create_cronjob(
            context           = context,
            name              = exp_id,
            filename          = data['filename'],
            crontab           = data['crontab'],
            command           = cmd_array,
            image             = settings['image'],
            p2_store_endpoint = settings['p2_store_endpoint'],
            namespace         = settings['namespace'],
            service_account   = settings['service_account']
            )
        log.info("Cronjob {} created".format(cronjob_name))
        return cronjob_name
    except Exception as err:
            log.error(str(err))
            return JSONResponse(
                content='{"Error": "Error creating cronjob"}',
                status_code=500
                )


@app.delete('/agent/k8s/cronjob/{context}/{id}', status_code=200)
def deleteCronjob(context, id):
    k8s_obj = k8s.Cron(context=context, namespace=settings['namespace'])
    deleted_id = k8s_obj.delete_cronjob(id)
    if deleted_id is None:
        log.error("Error deleted cronjob {}".format(id))
        return JSONResponse(status_code=500)
    return deleted_id


#-------------------------------------------------------------------------
# Jobs management
#-------------------------------------------------------------------------
@app.post('/agent/k8s/job/{context}/{name}', status_code=201)
async def createJob(context:str, name:str, request: Request):
    """ payload:
            image,
            command,
            name,
            service_account
            """

    job_obj = jobs.Job(context, settings['namespace'])
    data = await request.json()
    return job_obj.create_job(**data)


@app.get('/agent/k8s/jobs/{context}')
async def getJobs(context):
    job_obj = jobs.Job(context, settings['namespace'])
    return job_obj.get_jobs()
