import os
import logging
from datetime import datetime
from jinja2 import Environment, FunctionLoader, select_autoescape
from jinja2 import BaseLoader, TemplateNotFound
from pprint import pformat

from p2.store.client import Client
import p2.agent.experiment

class P2Loader(BaseLoader):

    def __init__(self, p2_store_url):
        self.p2_store_url = p2_store_url

    def get_source(self, environment, template):
        client = Client(self.p2_store_url)
        log = logging.getLogger(__name__)
        templates = client.select(
            table='experiments',
            fetch=['data', 'last_update_time'],
            filter={
                'filename': template,
                'type': 'template'
            })

        if templates is None:
            raise TemplateNotFound(template)
        source = templates[0]['data']
        updated = datetime.timestamp(
            datetime.fromisoformat(templates[0]['last_update_time'])
        )
        log.debug(f"Template:\n{template}")
        log.debug(f"Last updated: {updated}")

        return source, template, updated


class Runner(p2.agent.experiment.Runner):
    """A class to run an experiment based on a template.

    Templates are defined using Jinja2.
    Template parameters are of 2 kinds:
    - static parameters: valid for all experiments result
    of template processing.
    - dynamic parameters: experiment specific, different
    for each target the template is processed against.

    Static parameters are stored along with the template or the
    test campaign. If set for the test campaign, they override the
    template ones.
    Dynamic ones are passed to this class constructor at runtime
    and are defined by the set of targets that satisfy the
    filter set on the test campaign for the target resources

    This class extends the experiment Runner one, adding some
    logic to handle the template before running the experiment.

    The experiment result of template parsing is stored back into
    the p2.store.experiments table with the session id as filename.
    """

    def run(self):
        """ Workflow:
        - get template data
        - parse template into experiment json
        - store json experiment into experiments table
        - super().run()
        """
        template_rec = self.client.select(
            table='experiments',
            filter={
                'filename': self.filename,
                'type': 'template'
            })
        if template_rec is None:
            raise Exception(f"Template {self.filename} not found")

        static_params=dict()
        if self.campaign_id:
            campaign = self.client.select(
                table='test_campaigns',
                filter={
                    'id': self.campaign_id
                })
            static_params = campaign[0]['template_params']
        else:
            static_params = template_rec[0]['profile']

        env = Environment(
            loader=P2Loader(self.p2_store_url),
            autoescape=select_autoescape(['json']),
            trim_blocks=True
        )

        session = self.client.select(
            table='exp_session',
            filter={'exp_id':self.exp_id}
        )
        if session is None:
            raise Exception(f"Session {self.exp_id} not found")

        if session[0]['status'] != 'scheduled':
            raise Exception(f"Session {self.exp_id} not in scheduled status, aborting...")

        target_params = dict()

        if session[0]['resource_id'] is not None:
            self.log.info(f"Getting target profile from resource id: {session[0]['resource_id']}...")
            resource = self.client.select(
                table='resources',
                filter={'id': session[0]['resource_id']}
                )
            if len(resource) == 1:
                target_params = {'target': resource[0]['profile']['metadata']}
            else:
                self.log.error(f"Resource {session[0]['resource_id']} not found, reverting to session profile...")
                target_params = session[0]['profile']
        else:
            target_params = session[0]['profile']

        self.log.debug("Target params: {}".format(pformat(target_params)))
        t = env.get_template( self.filename )
        experiment = t.render( {**static_params, **target_params} )

        template_rec[0]['filename'] = self.exp_id
        template_rec[0]['data'] = experiment
        template_rec[0]['type'] = 'experiment'

        # store new experiment back into p2.store...
        res = self.client.add(
            table='experiments',
            record=template_rec[0]
        )
        if res is None:
            raise Exception(
                f"Can't add experiment {self.exp_id} out of template"
            )

        # and now execute it...
        self.filename = self.exp_id
        super().run()
